#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# neural network class definition
import scipy.special
import numpy 
# library for plotting arrays
import matplotlib.pyplot
# helper to load data from PNG image files# helpe 
import imageio
# glob helps select multiple files using patterns
import glob

class neuralNetwork : 
    
    # initialise the neural network 
    def __init__(self, inputnodes, hiddennodes, outputnodes, learningrate) : 
        
        #set number of nodes in each input , hidden , output
        #初始化网络，设置输入层，中间层，和输出层节点数
        self.inodes = inputnodes
        self.hnodes = hiddennodes
        self.onodes = outputnodes

        # link weight matrices, wih and who 
        # weights inside the arrays are w_ i_ j, where link is from node i to node j in the next layer 
        # w11 w21 
        # w12 w22 etc 
        # 初始化权重矩阵，我们有两个权重矩阵，一个是wih表示输入层和中间层节点间链路权重形成的矩阵一个是who,表示中间层和输出层间链路权重形成的矩阵
        self. wih = numpy.random.normal( 0.0, pow( self. hnodes, -0.5), (self. hnodes, self. inodes))
        self. who = numpy.random.normal( 0.0, pow( self. onodes, -0.5), (self. onodes, self. hnodes))

        # learning rate 
        self.lr = learningrate

        # activation function is the sigmoid function
        # 设置激活函数的反函数
        self.activation_function = lambda x:scipy.special.expit(x) 

        pass 
    
    # train the neural network 
    def train(self, inputs_list, targets_list) : 

        # convert inputs list to 2d array
        #根据输入的训练数据更新节点链路权重
        #把inputs_list, targets_list转换成numpy支持的二维矩阵.T表示做矩阵的转置
        inputs = numpy.array(inputs_list, ndmin=2).T
        targets = numpy.array(targets_list, ndmin=2).T
        
        # calculate signals into hidden layer
        #计算信号经过输入层后产生的信号量
        hidden_inputs = numpy.dot(self.wih, inputs)
        # calculate the signals emerging from hidden layer
        #中间层神经元对输入的信号做激活函数后得到输出信号
        hidden_outputs = self.activation_function(hidden_inputs)
        
        # calculate signals into final output layer
        #输出层接收来自中间层的信号量
        final_inputs = numpy.dot(self.who, hidden_outputs)
        # calculate the signals emerging from final output layer
        #输出层对信号量进行激活函数后得到最终输出信号
        final_outputs = self.activation_function(final_inputs)
        
        # output layer error is the (target - actual)
        #计算误差
        output_errors = targets - final_outputs
        # hidden layer error is the output_errors, split by weights, recombined at hidden nodes
        hidden_errors = numpy.dot(self.who.T, output_errors) 
        
        #根据误差计算链路权重的更新量，然后把更新加到原来链路权重上
        # update the weights for the links between the hidden and output layers
        self.who += self.lr * numpy.dot((output_errors * final_outputs * (1.0 - final_outputs)), numpy.transpose(hidden_outputs))
        
        # update the weights for the links between the input and hidden layers
        self.wih += self.lr * numpy.dot((hidden_errors * hidden_outputs * (1.0 - hidden_outputs)), numpy.transpose(inputs))
        pass 
    
    # query the neural network 
    def query(self, inputs_list) : 

        #根据输入数据计算并输出答案
        # convert inputs list to 2d array
        inputs = numpy.array(inputs_list, ndmin=2).T

        #计算中间层从输入层接收到的信号量
        # calculate signals into hidden layer
        hidden_inputs = numpy.dot(self.wih, inputs)

        #计算中间层经过激活函数后形成的输出信号量
        # calculate the signals emerging from hidden layer
        hidden_outputs = self.activation_function(hidden_inputs)
        
        #计算最外层接收到的信号量
        # calculate signals into final output layer
        final_inputs = numpy.dot(self.who, hidden_outputs)
        # calculate the signals emerging from final output layer
        final_outputs = self.activation_function(final_inputs)
        
        return final_outputs
     
